# -*- encoding: utf-8 -*-

############################################################################################
#
#  FILE AUTOMATICALLY GENERATED BY JFML FOR RUNNING A FUZZY SYSTEM IN A RASPBERRY PI BOARD 
#  YOU CAN EDIT THIS FILE ACCORDING TO YOUR SPECIFICATION
#
# VARIABLES - SENSORS
#      1: proximity -- HC-SR04 for Raspberry
#
##############################################################################################


########## LIBRARIES ##############
import re # to split function
from time import sleep # to sleep
import RPi.GPIO as GPIO
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
##########################################

########## SENSOR LIBRARIES ##############
from gpiozero import DistanceSensor # http://gpiozero.readthedocs.io/en/stable/index.html
##########################################

########## CONNECTION LIBRARIES ##############
import serial # included in linux system or pip3 install pyserial
##########################################

################# UART PORT ###########################
client = serial.Serial(
	port='/dev/rfcomm0',
	baudrate = 9600,
	parity=serial.PARITY_NONE,
	stopbits=serial.STOPBITS_ONE,
	bytesize=serial.EIGHTBITS,
	timeout=1)
#########################################################


####################### MAP FUNCTION ####################
def map(value, istart, istop, ostart, ostop):
	return ostart + (ostop - ostart) * ((value - istart) / (istop - istart))
#########################################################

############ MAPPING VARIABLES ################
numVariables = 1

pin_proximity = 25
index_proximity = 0
#################################################

######### COMMUNICATION PROTOCOL DEFINITIONS ############
BAUD_RATE = 9600
MAX_MESSAGE_SIZE = 200 

START_READ = "r" 
START_WRITE = "w" 
STOP = "x" 
ACK = "ACK" 

start_read = False #boolean variable for start reading in Raspberry 
start_write = False #boolean variable for start writing in Raspberry 
stop_communication = False #boolean variable for stop communication with Raspberry 

read_function = [None] * numVariables 
write_function = [None] * numVariables
write_value = [None] * numVariables
#########################################################



################ ULTRASONIC SENSOR HC SR04 ####################
TRIGGER_PIN = 25
ECHO_PIN = 5
MAX_DISTANCE = 255.0
distanceSensor = DistanceSensor(echo=ECHO_PIN, trigger=TRIGGER_PIN, max_distance=2)
#########################################################


######## SWITCH FUNCTION TO READ FUNCTIONS
def switch_read(argument):
	global val
	switcher = {
		index_proximity: read_proximity()
	}
	val = str(switcher.get(argument))

######## SWITCH FUNCTION TO WRITE FUNCTIONS
def switch_write(argument,value):
	global val
	switcher = {
	}
	val = str(switcher.get(argument))

################ MAIN FUNCTION ################
def main():
	while True:
		numActions = processingActionPacket()

		#read from Raspberry and send data to Java
		if(start_read and not stop_communication):
			readValuesFromRaspberry(numActions)

		#write from Java to raspberry
		if(start_write and not stop_communication):
			writeValuesFromJava(numActions)

################ PROCESSING ACTION PACKET ################
def processingActionPacket():
	global client
	global start_read
	global start_write
	global stop_communication
	global write_function
	global write_value
	first_command = False

	data = ""
	command = []
	num_actions = 0 #number of pins requested in the message


	data = client.recv(MAX_MESSAGE_SIZE).decode()

	if data:
		command = re.split('&|:', data)
		if not ":" in data:
			count = len(command)-1
		else:
			count = data.count(':')

		#if this is the first command
		if(not first_command):
			if(command[0]==START_READ):
				start_read=True
			if(command[0]==START_WRITE):
				start_write=True
			if(command[0]==STOP):
				stop_communication=True
			else:
				stop_communication=False
			first_command = True

		for i in range(count):
			if(start_read == True):
				read_function[i] = int(command[i+1])
				num_actions += 1
			if(start_write == True):
				write_function[i] = int(command[i+1])
				write_value[i] = float(command[i+2])
				num_actions += 1

		command = []
	return num_actions

################ READ VALUES FROM RASPBERRY ################
def readValuesFromRaspberry(num):
	global start_read
	global client
	global val

	message=""

	for i in range(num):
		switch_read(read_function[i])

		message += str(read_function[i]) # variable index in JFML
		message += "/"
		message += val #value of the read

		if(i<num-1):
			message += "#"


		sleep(0.01) # delay of 10 miliseconds in between reads for stability
	message += "\n"

	client.write(message.encode())
	start_read = False

################ WRITE VALUES FROM JAVA ################
def writeValuesFromJava(num):
	global start_write
	global client
	global val

	message=""

	for i in range(num):
		switch_write(write_function[i], write_value[i])
		message += str(write_function[i]); #variable index in JFML
		message += "/"
		message += val #value of the write (ACK)

		if(i<num-1):
			message += "#"

	message += "\n"
	client.write(message.encode())
	start_write = False

def read_proximity():
	#######################################################
	#Enter your own code in the read function if you consider
	#to obtain 'val' value to return
	#######################################################
	global distanceSensor
	val = round((distanceSensor.distance * 100),2)

	return str(val)
if __name__== "__main__":
	main()
